package cn.kdan.utils;

import lombok.extern.slf4j.Slf4j;

import javax.mail.*;
import java.io.*;
import java.util.Base64;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author ComPDF-Bob 2024-06-11
 **/
@Slf4j
public class EmailUtil {

    /**
     * 获取邮件正文
     * @param part 邮件信息
     * @return 正文
     * @throws MessagingException
     * @throws IOException
     */
    public static String getTextFromMessage(Part part) throws MessagingException, IOException {
        if (part.isMimeType("text/plain")) {
            return (String) part.getContent();
        } else if (part.isMimeType("text/html")) {
            // 获取 HTML 内容并移除 HTML 标签
            String html = (String) part.getContent();
            return stripHtmlTags(html);
        } else if (part.isMimeType("multipart/*")) {
            Multipart multipart = (Multipart) part.getContent();
            StringBuilder result = new StringBuilder();
            // 遍历每个部分，确保只添加非空的内容
            for (int i = 0; i < multipart.getCount(); i++) {
                BodyPart bodyPart = multipart.getBodyPart(i);
                result.append(getTextFromMessage(bodyPart));
//                if (bodyPart.isMimeType("text/plain")) {
//                    result.append(bodyPart.getContent());
//                }
            }
            return result.toString();
        }
        return "";
    }

    // 移除 HTML 标签的方法
    private static String stripHtmlTags(String html) {
        return html.replaceAll("<[^>]*>", "");
    }

    /**
     * 从邮件正文中提取目标邮箱地址
     *
     * @param body 邮件正文
     * @return 目标邮箱地址
     */
    public static String extractTargetEmail(String body) {
        Pattern pattern = Pattern.compile("([\\w.-]+@[\\w.-]+\\.[\\w]+)");
        Matcher matcher = pattern.matcher(body);
        if (matcher.find()) {
            return matcher.group(1);
        }
        return "";
    }

    /**
     * 获取附件
     * @param bodyPart
     * @return 文件
     * @throws MessagingException
     * @throws IOException
     */
    public static File saveAttachment(BodyPart bodyPart) throws MessagingException, IOException {
        InputStream is = bodyPart.getInputStream();
        File file = new File(System.currentTimeMillis() + bodyPart.getFileName());
        try (FileOutputStream fos = new FileOutputStream(file)) {
            byte[] buffer = new byte[4096];
            int bytesRead;
            while ((bytesRead = is.read(buffer)) != -1) {
                fos.write(buffer, 0, bytesRead);
            }
        }
        return file;
    }

    /**
     * 保存邮件中的图片
     * @param part 邮件信息
     * @param files 保存图片的文件列表
     * @throws MessagingException
     * @throws IOException
     */
    public static void saveAttachmentFiles(Part part, List<File> files) throws MessagingException, IOException {
        // 有附件或图片的邮件只需要保存一个
        if (files.size() > 0) {
            return;
        }
        if (part.isMimeType("multipart/*")) {
            Multipart multipart = (Multipart) part.getContent();
            for (int i = 0; i < multipart.getCount(); i++) {
                BodyPart bodyPart = multipart.getBodyPart(i);
                if (bodyPart.isMimeType("application/pkcs7-signature")) {
                    // 跳过签名
                    continue;
                }
                if (bodyPart.isMimeType("multipart/related") || bodyPart.isMimeType("multipart/MIXED")
                || bodyPart.isMimeType("multipart/alternative") || bodyPart.isMimeType("multipart/signed")
                || bodyPart.isMimeType("multipart/report")) {
                    saveAttachmentFiles(bodyPart, files); // 递归处理嵌套的multipart/related
                } else if (Part.ATTACHMENT.equalsIgnoreCase(bodyPart.getDisposition()) || Part.INLINE.equalsIgnoreCase(bodyPart.getDisposition()) || isInlineImage(bodyPart)) {
                    saveFile(bodyPart, files);
                }
            }
        }
    }

    /**
     * 判断是否是内嵌图片
     * @param bodyPart 图片信息
     * @return
     * @throws MessagingException
     */
    private static boolean isInlineImage(BodyPart bodyPart) throws MessagingException {
        if ((bodyPart.isMimeType("image/*") || bodyPart.isMimeType("video/*")) && bodyPart.getFileName() == null) {
//        if ((bodyPart.isMimeType("image/*")) && bodyPart.getFileName() != null) {
            return true;
        }
        return false;
    }

    /*private static void saveAttachment(Part part, List<File> files) throws IOException, MessagingException {
        String[] contentIdHeader = part.getHeader("Content-ID");
        String contentId = (contentIdHeader != null && contentIdHeader.length > 0) ? contentIdHeader[0] : null;
        if (contentId != null) {
            contentId = contentId.replaceAll("[<>]", ""); // 移除尖括号
        }
        String fileName = part.getFileName();
        if (fileName != null) {
            InputStream is = part.getInputStream();
            File file = new File((contentId != null ? contentId : "") + System.currentTimeMillis() + fileName);
            OutputStream os = new FileOutputStream(file);
            byte[] buffer = new byte[1024];
            int bytesRead;
            while ((bytesRead = is.read(buffer)) != -1) {
                os.write(buffer, 0, bytesRead);
            }
            os.close();
            is.close();
            System.out.println("Attachment saved: " + file);
            files.add(file);
        }
    }*/

    /**
     * 保存图片
     * @param bodyPart 图片信息
     * @param imageFiles 保存图片的文件列表
     * @throws IOException
     * @throws MessagingException
     */
    private static void saveFile(BodyPart bodyPart, List<File> imageFiles) throws IOException, MessagingException {
        InputStream inputStream = bodyPart.getInputStream();
        String fileName = getFileNameFromBodyPart(bodyPart);
        File file = new File(fileName);
        try (FileOutputStream outputStream = new FileOutputStream(file)) {
            byte[] buffer = new byte[4096];
            int bytesRead;
            while ((bytesRead = inputStream.read(buffer)) != -1) {
                // 有附件或图片的邮件只需要保存一个
                if (imageFiles.size() > 0) {
                    return;
                }
                outputStream.write(buffer, 0, bytesRead);
            }
            outputStream.close();
            inputStream.close();
        }
        log.info("Saved File to {}", file.getAbsolutePath());
        imageFiles.add(file);
    }

    /**
     * 从BodyPart中获取文件名
     * @param bodyPart BodyPart
     * @return 文件名
     * @throws MessagingException
     */
    private static String getFileNameFromBodyPart(BodyPart bodyPart) throws MessagingException, UnsupportedEncodingException {
        String fileName = bodyPart.getFileName();
        // 判断是否是base64编码
        if (isMimeEncoded(fileName)) {
            log.info("处理文件名base64编码");
            fileName = decodeMimeEncodedFilename(fileName);
        }
        String[] contentIdHeader = bodyPart.getHeader("Content-ID");
        String contentId = (contentIdHeader != null && contentIdHeader.length > 0) ? contentIdHeader[0] : null;
        if (contentId != null) {
            contentId = contentId.replaceAll("[<>]", ""); // 移除尖括号
        }
        return (contentId != null ? contentId : "") + System.currentTimeMillis() + fileName;
    }

    public static boolean isMimeEncoded(String str) {
        // 正则表达式匹配 MIME 编码的开始和结束标记
        Pattern pattern = Pattern.compile("^=\\?.+\\?.\\?.+\\?=$");
        return pattern.matcher(str).matches();
    }

    public static String decodeMimeEncodedFilename(String encodedFilename) {
        // 正则表达式匹配 MIME 编码的开始和结束标记
        Pattern pattern = Pattern.compile("=\\?([^?]+)\\?([BbQq])\\?([^?]+)\\?=");

        StringBuilder decodedFilenameBuilder = new StringBuilder();
        Matcher matcher = pattern.matcher(encodedFilename);

        while (matcher.find()) {
            String charset = matcher.group(1); // 编码字符集
            String encodingType = matcher.group(2); // 编码类型 (B 或 Q)
            String encodedContent = matcher.group(3); // Base64 或 Quoted-Printable 编码的内容

            // 只处理 Base64 编码的内容
            if ("B".equalsIgnoreCase(encodingType)) {
                try {
                    byte[] decodedBytes = Base64.getDecoder().decode(encodedContent);
                    String decodedPart = new String(decodedBytes, charset);
                    decodedFilenameBuilder.append(decodedPart);
                } catch (Exception e) {
                    e.printStackTrace(); // 处理解码异常
                }
            }
        }

        return decodedFilenameBuilder.toString();
    }
}
